/*
 This file is one part of two, that together make QOpenCamWidget, 
 a Qt 4 widget that displays video input from a webcam.

 Copyright (C) 2009 Jon Langseth

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation in its version 2
 of the License.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "qopencamwidget.h"

// ------------------------------------------------------------- //
// Constructor and Destructor
// ------------------------------------------------------------- //

/*!
 * \brief Consctructs a QWidget based widget for displaying video
 *        coming from an OpenCV capture source
 *
 * Including webcam data in a Qt application can be problematic,
 * at least as long as Phonon does not support webcams, and the
 * Phonon GStreamer backend only supports simple pipelines.
 *
 * This class solves the complexity of adding a webcam view,
 * by using the cross-platform available OpenCV library.
 *
 * Limitations, i.e. reasons to read this code and reimplement, are: 
 *
 * \li saving or streaming video is not really available (unless you do 
 *     repeated timer-triggered connections to the startSnap slot).
 * \li this is a crude and simple implementation, created to solve
 *     the problem of creating a "view-and-shoot" camera app.
 *     If your needs are more complex than that, read the code,
 *     and reimplement.
 *
 * Note that even tough the main file here totals some ~250 lines,
 * only about 90 of those are actual code lines, and and even those
 * contain a lot of "air"...
 *
 * A brief summary of how to use this class:
 * \code
 * QOpenCamWidget *cw = new QOpenCamWidget(this);
 * if ( cw->grabCapture(-1) ) {
 *   cw->startCapture();
 * }
 * connect( cw, SIGNAL(imageReady(QImage)), this, SLOT(saveImage(QImage)));
 * 
 * QPushButton *trigger = new QPushButton(this);
 * trigger->setText("Take picture");
 * connect( trigger, SIGNAL(clicked()), cw, SLOT(startSnap()));
 *
 * \endcode
 *
 * \param *parent The parent widget containing this widget, defaults to NULL.
 *
 **/	
QOpenCamWidget::QOpenCamWidget(QWidget *parent) 
	: QWidget(parent) 
{
	// Setting sane default values (i.e. NULL) for 
	// private CvCapture *nextFrame  and
	// private QTimer    *frametimer from class definition
	nextFrame = NULL;
	frametimer = NULL;
}

QOpenCamWidget::~QOpenCamWidget(void)
{
	cvReleaseCapture( &capture );
}
// ------------------------------------------------------------- //
// Public methods (not signals/slots)
// ------------------------------------------------------------- //

/*!
 * \brief A paint event is a request to repaint all or part of a widget. 
 *
 * It can happen for one of the following reasons:
 *
 *     \li repaint() or update() was invoked,
 *     \li the widget was obscured and has now been uncovered, or
 *     \li many other reasons.
 *
 * QOpenCamWidget uses the paintEvent to draw each frame onto the screen.
 * The paintEvent itself is regularily triggered by explicit update()
 * calls in QOpenCamWidget::grabFrame().
 *
 **/
void 
QOpenCamWidget::paintEvent ( QPaintEvent * event )
{
	QPainter * paint = new QPainter;
	paint->begin(this);
		
	if ( nextFrame )
	{
		// To make the widget as blazingly fast as possible
		// we output the last captured frame direclty onto
		// the widget area.
		paint->drawImage(event->rect(), *nextFrame);
	}
	else
	{
     		paint->drawText(event->rect(),"No data, check/test camera");
	}
	paint->end();
	// Clean up..
	delete(paint);
}


/*!
 * \brief Grabs an OpenCV video capture source
 *
 * By grabbing a source, it is meant to open the capture source,
 * and have it ready to start streaming/capturing frames.
 * Returns true on success, false on error. The grabCapture
 * is separated from the constructor and/or frame-grabbing, so that you
 * may do the error-checking you really should do before proceeding.
 *
 * \param source The OpenCV capture source enumeration index to open
 *
 **/
bool 
QOpenCamWidget::grabCapture(int source)
{
	capture = cvCaptureFromCAM(0);
	if (!capture)
	{
		qDebug() << "QOpenCamWidget::grabCapture(" << source << ") failed";
		return false;
	}
        cvGrabFrame(capture); // Grab a single frame, do resizing based on it.
        IplImage *image = cvRetrieveFrame(capture);
        QSize t_size = QSize(image->width,image->height);

	this->setMinimumSize(t_size);
	this->setMaximumSize(t_size);

	return true;
}

/*!
 * \brief Starts up grabbing of video frames
 *
 * The actual grabbing and displaying of video frames is performed by
 * a QTimer triggering the SLOT QOpenCamWidget::grabFrame().
 * startCapture() sets up the timer running this captureFrame loop.
 *
 * The SLOT QOpenCamWidget::startSnap() is used to get image frames
 * out from the widget for other uses, like saving or processing.
 * This function relies on the timer created and configured by
 * startCapture(), and as such, this function is the only permitted
 * way to start the actual capture/streaming of video from the source.
 *
 **/
void 
QOpenCamWidget::startCapture(void)
{
	frametimer = new QTimer(this);
        frametimer->start(50);
        connect(frametimer,SIGNAL(timeout()), this,SLOT(grabFrame()));
}

/*!
 * \brief Converts from the OpenCV IplImage data structure to a QImage
 *
 * OpenCV uses a data strcuture calles IplImage, optimized for 
 * computer vision image processing tasks. This code was adapted
 * from kcamwidget.cpp, part of the KDE SVN at
 * playground/multimedia/kcam/kcamwidget.cpp
 *
 * In regard that the IplImage can be forced into a format
 * that aligns well with a RBG888-format, the conversion
 * becomes one of the shortes, simples IplImage->QImage I've seen.
 *
 * \param *img The IplImage to be converted to a QImage.
 *
 **/
QImage* 
QOpenCamWidget::Ipl2QImage(IplImage *img)
{
	cvConvertImage(img,img, CV_CVTIMG_SWAP_RB);
	QImage * qimage = new QImage(
		reinterpret_cast<uchar*>(img->imageData),
		img->width, img->height,
		3* img->width, QImage::Format_RGB888);
	return qimage;

}
// ------------------------------------------------------------- //
// Public SLOTS
// ------------------------------------------------------------- //

/*!
 * \brief Grabs a frame and causes an update() when triggered.
 *
 * This is the SLOT that actually reads the video source and
 * causes the widget to display live video. Preferably this
 * slot will never be called my any other signal that a timeout()
 * on the frametimer, which is controlled by QOpenCamWidget::startCapture()
 *
 **/
void QOpenCamWidget::grabFrame(void)
{
	if ( !capture ) { qDebug() << "Capture device not ready!"; return; }

	cvGrabFrame(capture);

	IplImage *iplimage = cvRetrieveFrame(capture);
	if (iplimage)
	{
		nextFrame = Ipl2QImage(iplimage);
	}
	else
	{
		nextFrame =  NULL;
	}

	update();
}

/*!
 * \brief Trigger this slot to save a frame from the widget.
 *
 * When this slot is triggered, the widgets capture loop is temporarily
 * stopped, and the last displayed frame is "captured", and made
 * available through the emitting of the class imageReady SIGNAL.
 *
 * It is possible, though I would not recommend, to use repeated
 * triggering of this slot to do repeated frame-capture, and thus
 * make a form of "Animation" or "Video" capture.
 *
 **/
void QOpenCamWidget::startSnap(void)
{
	if ( frametimer ) {
		if (frametimer->isActive())
		{
			frametimer->stop();
			emit imageReady(QImage(*nextFrame));
			frametimer->start();
		}
	}
}

