/*
 QOpenCamWidget demo application main window (view.c)
 This file is part of a demonstration application, illustrating
 how to use the QOpenCamWidget, a Qt 4 widget that displays
 video input from a webcam, along with a snapshot button.

 Copyright (C) 2009 Jon Langseth

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation in its version 2
 of the License.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "view.h"
#include "qopencamwidget.h"

// The View class is a QMainWindow, used to display our QOpenCamWidget
// The constructor of the View takes care of setting up the Widget,
// and handles the result of a user clicking (or activating) the
// "Take snapshot" button.

View::View( QWidget *parent)
	: QMainWindow(parent)
{
	
	QWidget *cw = new QWidget();
	QVBoxLayout *layout = new QVBoxLayout();

	// This button will be used to trigger a snapshot later on..
	QPushButton *trigger = new QPushButton(this);
	trigger->setText("Take picture");

	// Create the widget, setting the Main window (this) as its parent.
	QOpenCamWidget *cwcam = new QOpenCamWidget(this);
	// bool QOpenCamWidget::grabCapture(int source) tries to get a
	// video capture handle from OpenCV, using the source number
	// as the device to open. Here I use -1 as the number, meaning
	// "give me any supported and available video devica you can find".
	if ( cwcam->grabCapture(-1) )
	{
		// grabCapture() returns true on success. This means that
		// we have a capture device available, and can start streaming
		// video from the device to display on the widget.
		
		// Video output does not start until QOpenCamWidget::startCapture()
		// is explicitly called.
		cwcam->startCapture();
	}

	// The QOpenCamWidget does not provide a video stream for further
	// processing, but it does provide a snapshot of the stream if
	// the SLOT void QOpenCamWidget::startSnap(void) is triggered.
	// In this demo app, the slot is connected to the widgets
	// internal "Take snapshot" trigger button, but the SLOT is
	// public, and may easily be triggered from "outside".
	//
	// When startSnap() is triggered, it will use a SIGNAL to return
	// its image, if the aquisition was successful. The image
	// will be available through QOpenCamWidget::imageReady(Qimage);
	// In this sample, I hook that signal up to a slot on this QMainWindow,
	// that saves the image, and exits the application.
	connect( cwcam, SIGNAL(imageReady(QImage)), this, SLOT(saveImage(QImage)));

	// To trigger the startSnap(), we hook our previously created
	// pushbutton to that SLOT.
	connect( trigger, SIGNAL(clicked()), cwcam, SLOT(startSnap()));

	// Add it all up, and we are ready to run.
	layout->addWidget(cwcam);
	layout->addWidget(trigger);
	cw->setLayout(layout);
	this->setCentralWidget(cw);
}

View::~View(void)
{
}

void View::saveImage(QImage image)
{
	// This is the SLOT that recieves QOpenCamWidget::imageReady(QImage) SIGNAL.
	// To keep the example as simple as possible, I do as little as possible..
	qDebug() << "Recieved an image of size " 
		<< image.width() << "x" << image.height();
	image.save("snapshot.png");

	QApplication::exit();
}
